/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.PagedTableViewlet");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.util.ResizeNotifier");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {
	
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ResizeNotifier = jazz.util.ResizeNotifier;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

dojo.declare("com.ibm.team.dashboard.web.ui.PagedTableViewlet",
	Viewlet, {
	
	// summary:
	//		An abstract subclass of Viewlet that provides paging support.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		Provides several functions that can be overriden to create a paged viewlet.
	//		The pages of the viewlet show tabular content and the consumer is reponsible
	//		for providing functions to fetch and render the data. In order for the viewlet
	//		to function, the methods that must be implemented are: fetch and renderEntry.
	//		If the viewlet is zoomable, then the renderDetails function must also be
	//		implemented.
	//
	//		The viewlet supports zoomable subclasses by allowing more detailed fetches to
	//		be performed when the user zooms in. Calls to the fetch function will provide
	//		flags to indicate if basic or detailed data should be returned. When zoomed,
	//		two rows will be rendered for each entry. The first will be generated by the
	//		renderEntry function and the second by the renderDetails function.
	//
	//		Since all table cells are generated by calls to functions implemented by the
	//		subclass, consumers are free to style and structure the table however they
	//		desire. This can be done by specifying attributes and css classes on the DOM
	//		nodes returned from the render functions.
	//
	//		Consumers overriding any Viewlet methods must call this.inherited(arguments)
	//		to ensure they do not override any PagedTableViewlet functionality.
	// since:
	//		0.7
	
	/* TO BE OVERRIDEN BY SUBCLASSES IF NEEDED */
	
	//		Used to customize the message that will be shown to the user when the viewlet is
	//		loading. The content will be applied as text, so html tags should not be used.
	loadingText: messages.pagedTableLoading,
	
	//		A string which will be applied as a css class to the content area of the viewlet.
	//
	//		NOTE: The DOM structure and CSS classes of the PagedTableViewlet are provisional
	//		and subject to change at future times without warning.
	contentClass: null,
	
	onDestroy: function() {
		// summary:
		//		Called in the viewlet's destroy method. Can be used to perform required cleanup.
	},
	
	/* int */ getItemsPerPage: function() {
		// summary:
		//		Will be called by the viewlet to get the number of items to show per page. The
		//		default implementation returns 10. This method can be overriden to perform
		//		some logic to determine the number of items per page, or simply to return a
		//		different value. A common practice is to declare a viewlet preference to allow
		//		the user to configure the number of items per page and to implement this
		//		method to simply return the value of the preference.
		return 10;
	},
	
	/* int */ getResultCount: function() {
		// summary:
		//		Will be called by the viewlet to get the number of results for the current query.
		//		If the viewlet has no valid query or target, 0 should be returned as if there
		//		were simply no results. This will trigger a call to the getNoResultsMessage
		//		method which allows the viewlet to be populated with a helpful message to guide
		//		the user about its configuration. If the viewlet is configured correctly, but the
		//		number of results is not known yet, -1 should be returned. This will trigger a
		//		a call to the fetch method.
		return 0;
	},
	
	/* Node */ getNoResultsMessage: function() {
		// summary:
		//		Called when getResultCount returns 0 to get a message for the user.
		// description:
		//		This method can be overriden to customize what is shown to the user when there
		//		are no results. It can also be used to guide the user if the viewlet is not
		//		configured correctly. The return type should be a dom node.
		return document.createTextNode(messages.pagedTableNoResults);
	},
	
	fetch: function (/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		// summary:
		//		This function will be called when more entries are required to render a given page.
		// description:
		//		Flags are passed to the function to indicate the type of content that is required.
		//		The viewlet should use these flags to determine what sort of requests to make to the
		//		server and to determine what to include in the result.
		//
		//		To allow for asynchronous requests, nothing is returned from this method. Instead
		//		the data should be provided by calling the provided ServiceResponseHandler's
		//		callback function and passing in the result object. This should be an object with
		//		one or two attributes. If the entries flag is specified it must have an a entries
		//		attribute and if the details flag is specified, it must have a details attribute.
		//		Each attribute on the result object should be an array of objects. The objects can
		//		take whatever form the viewlet likes and they will be passed to the renderEntry
		//		and renderDetails functions for rendering. Thus, they will only be consumed by the
		//		viewlet implementor, not by the base class. Each array in the result must be equal
		//		to or shorter than the count parameter in length. They should only be shorter in
		//		the case that the last page is being requested and there are not enough items left
		//		to fill them.
		// srh:
		//		The ServiceResponseHandler object to return the results to once they are ready.
		// startIndex:
		//		The index of the first item to fetch in the current query.
		// count:
		//		The number of items to fetch.
		// entries:
		//		A flag indicating that the entry data should be included. This is the data used to
		//		render the main row for each item.
		// details:
		//		A flag indicating that the detail data should be included. This is the data used to
		//		render the details for an item when the viewlet is zoomed. Will only ever be true
		//		for zoomable viewlets.
		srh.doErrorCallback(messages.pagedTableFetchNotOverriden);
	},
	
	preRender: function(/* object[] */ entries) {
		// summary:
		//		Called immediately before rendering a page of data.
		// description:
		//		This method can be overriden by subclasses that need to perform some calculations
		//		that depend all the entries for a given page before it can render any of them
		//		individually.
		// entries:
		//		An array containing all the entries that are about to be rendered.
	},
	
	/* Node[] */ renderEntry: function(/* object */ entryData, /* object */ detailsData, /* boolean */ zoomed) {
		// summary:
		//		Called when an entry needs to be rendered into DOM nodes.
		// description:
		//		When the viewlet is rendering a page of data, it will call this function once per
		//		item that needs to be rendered. The function is responsible for reading the entry
		//		data that was originally generated by the fetch function and creating dom nodes
		//		out of it.
		//
		//		The return value must be an array of <td> DOM nodes. They will be used to populate the
		//		table row for the provided entry. Consumers may apply custom css styles and column
		//		spans to the returned cells.
		// entryData:
		//		The entry data object that was returned from a previous call to the fetch function.
		// detailsData:
		//		Will only be provided when the viewlet is zoomed. When zoomed, the value of this
		//		attribute is equal to the detail data returned from a previous call to the fetch
		//		funciton. The details correspond to the provided entryData. This is provided to allow
		//		consumers to change the rendering of the entry node when the viewlet is zoomed and
		//		to do so based on the information in the detailed data.
		console.error("renderEntry has not been overriden.");
	},
	
	/* Node[] */ renderDetails: function(/* object */ detailsData) {
		// summary:
		//		Called when the details for an entry need to be rendered (when the viewlet is zoomed).
		// description:
		//		When the viewlet is rendering a page of data while zoomed, it will call this function
		//		once per item that needs to be rendered. The function is responsible for reading the
		//		details data that was originally generated by the fetch function and creating dom nodes
		//		out of it.
		//
		//		The return value must be an array of <td> DOM nodes. They will be used to populate the
		//		details table row for the provided entry. Consumers may apply custom css styles and
		//		column spans to the returned cells.
		//
		//		The detail row for each item appear directly below their corresponding entry row in the
		//		table.
		// entries:
		//		The details data object that was returned from a previous call to the fetch function.
		return null;
	},
	
	/* TO BE CALLED BY SUBCLASSES */
	
	updateTotalPageFooter: function() {
		// summary:
		//		Recalculates the number of pages and updates the footer.
		// description:
		//		Should be called whenever the total number of results changes (e.g. when getResultCount
		//		will return a different value). This will recalculate the total number of pages and
		//		update the footer to reflect it.
		var pageCount = this._getPageCount()
		this._totalPagesFooter.innerHTML = pageCount;
		if (pageCount > 1)
			dojo.removeClass(this._footer, "hidden");
		else
			dojo.addClass(this._footer, "hidden");
	},
	
	reset: function(/* boolean */ refresh) {
		// summary:
		//		Resets the viewlet to page 1.
		// description:
		//		Should be called when the number of items per page changes or when the viewlet's query has
		//		changed. Contains a call to updateTotalPageFooter, so consumers do not need to call both
		//		functions.
		// refresh:
		//		A boolean parameter that indicates if a full viewlet refresh is required. If this is set to
		//		true, all cached items will be invalidated and a fetch call will be made to retrieve page
		//		1. The loading message will also be shown until the fetch completes. If refresh is set to
		//		false, a call to update will be made.
		this._currentPage = 1;
		this.updateTotalPageFooter();
		this._updateCurrentPageFooter();
		this._clearHeight();
		if (refresh) {
			this._setLoading(true);
			this.getSite().refresh();
		} else {
			this.update();
		}
	},
	
	setGlobalHeading: function(/* domNode */ node) {
		// summary:
		//		Places the provided DOM node at the top of the viewlet content area.
		// description:
		//		This can be used to provide a heading to the viewlet that will be visible regardless of
		//		which page is showing. It will still be visible if there are no results and will display
		//		above the no results message.
		//
		//		The global heading will be hidden when the loading message is displayed.
		// node:
		//		The DOM node to be used as the global heading.
		this.clearGlobalHeading();
		this._globalHeading.appendChild(node);
	},
	
	clearGlobalHeading: function() {
		// summary:
		//		Removes any previously created global heading.
		while (this._globalHeading.firstChild)
			this._globalHeading.removeChild(this._globalHeading.firstChild);
	},
	
	update: function() {
		// summary:
		//		Re-renders the current page of the viewlet.
		// description:
		//		Can trigger calls to fetch if the data for the current page is unavailable. Will trigger
		//		calls to renderEntry to perform the rendering. If the viewlet is zoomed, will also trigger
		//		calls to renderDetails.
		var isZoomed = this.isZoomed();
		if (isZoomed != this._previousIsZoomed) {
			if (isZoomed) {
				this._onZoom();
			}
			else {
				this._onUnzoom();
			}
			this._previousIsZoomed = isZoomed;
		}
		if (isZoomed)
			this._clearHeight();
		var itemsPerPage = this.getItemsPerPage();
		var resultCount = this.getResultCount();
		var startIndex = (this._currentPage - 1) * itemsPerPage;
		var entriesReady = true;
		var detailsReady = true;
		if (resultCount == -1) {
			entriesReady = false;
			detailsReady = !this.isZoomed();
		} else if (resultCount > 0 ){
			for (var i = startIndex; i < startIndex + itemsPerPage && i < resultCount; i++) {
				if (!this._entries[i]) 
					entriesReady = false;
				if (this.isZoomed() && !this._details[i]) 
					detailsReady = false;
				if (!entriesReady && (!this.isZoomed() || !detailsReady)) 
					break;
			}
		}
		var self = this;
		var serviceObject = {
			success: function(result) {
				if (result && result.entries && result.entries.length > 0) {
					for (var i = 0; i < result.entries.length; i++)
						self._entries[startIndex + i] = result.entries[i];
				}
				if (result && result.details && result.details.length > 0) {
					for (var i = 0; i < result.details.length; i++)
						self._details[startIndex + i] = result.details[i];
				}
				self._render();
				self._setLoading(false);
				self.getSite().setLoading(false);
				self.getSite().doneRefresh();
			},
			failure: function(error) {
				if (error && error.status == 403) {
					var span = document.createElement("span");
					span.className = "errorText";
					span.appendChild(document.createTextNode(messages.pagedTablePermissionDenied));
					self._showMessage(span);
					self._setLoading(false);
					self.getSite().setLoading(false);
					self.getSite().doneRefresh();
				}
				else {
					self._setLoading(false);
					self.getSite().setLoading(false);
					self.getSite().doneRefresh();
					self.getSite().error(error);
				}
			}
		};
		if (entriesReady && detailsReady) {
			this._render();
			this.getSite().doneRefresh();
		}
		else {
			this.getSite().setLoading(true);
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			this.fetch(srh, startIndex, itemsPerPage, !entriesReady, !detailsReady);
		}
	},
	
	/* INTERNAL FUNCTIONS */

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/PagedTableViewlet.html"),
	
	firstLinkImageSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/paged-table-viewlet/images/beginning.gif"),
	lastLinkImageSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/paged-table-viewlet/images/end.gif"),

	constructor: function() {
		this.next = messages.pagedTableNext;
		this.prev = messages.pagedTablePrev;
		this.first = messages.pagedTableFirst;
		this.last = messages.pagedTableLast;
	},
		
	init: function() {
		var nodes = I18nUtil.addMessage(this._pageNumbers, messages.pagedTablePageCount);
		this._pageText = nodes[0];
		this._currentPageFooter = nodes[1];
		this._totalPagesFooter = nodes[2];
		this._previousIsZoomed = false;
		this._currentPage = 1;
		this._updateCurrentPageFooter();
		this._loadingDiv.appendChild(document.createTextNode(this.loadingText));
		// set src on images, so their size is correct in IE
		this._firstLinkImage.src = this.firstLinkImageSrc;
		this._lastLinkImage.src = this.lastLinkImageSrc;
		if (this.contentClass) {
			dojo.addClass(this._contentArea, this.contentClass);
			dojo.addClass(this._globalHeading, this.contentClass);
		}
		this.connect(this.domNode.parentNode,'onmouseover',this._showControls);
		this.connect(this.domNode.parentNode,'onmouseout',this._hideControls);
		this.connect(this._firstLink,'onmouseover',this._onMouseOverButton);
		this.connect(this._firstLink,'onmouseout',this._onMouseOutButton);
		this.connect(this._lastLink,'onmouseover',this._onMouseOverButton);
		this.connect(this._lastLink,'onmouseout',this._onMouseOutButton);
		
		
		if (dojo.isIE) {
			// if you click fast in IE, it omits clicks
			this.connect(this._prevLink,'ondblclick',this._handlePrevPage);
			this.connect(this._nextLink,'ondblclick',this._handleNextPage);
		}
		this._dashboardResizeEvent = ResizeNotifier.register({context: this, callback: this._onResize});
		this.connect(this._footerStart,'onfocus',dojo.hitch(this,this._footerEntered,true));
		this.connect(this._footerEnd,'onfocus',dojo.hitch(this,this._footerEntered,false));
		this.connect(this._firstLink,'onblur',this._footerLinkBlur);
		this.connect(this._prevLink,'onblur',this._footerLinkBlur);
		this.connect(this._nextLink,'onblur',this._footerLinkBlur);
		this.connect(this._lastLink,'onblur',this._footerLinkBlur);
		this.connect(this._firstLink,'onfocus',this._footerLinkFocus);
		this.connect(this._prevLink,'onfocus',this._footerLinkFocus);
		this.connect(this._nextLink,'onfocus',this._footerLinkFocus);
		this.connect(this._lastLink,'onfocus',this._footerLinkFocus);
	},
	
	
	/*
	 * We have entered the footer so show the controls and then allow tabbing to continue.
	 */
	_footerEntered: function(forward){
		this._footerStart.tabIndex = -1;
		this._footerEnd.tabIndex = -1;
		this._showControls();
		if (forward) {
			if (this._currentPage == 1) {
				this._nextLink.focus();
			} else {
				this._firstLink.focus();
			}
		} else {
			if (this._currentPage == this._getPageCount()) {
				this._prevLink.focus();
			} else {
				this._lastLink.focus();
			}
		}
	},
	
	_footerLinkBlur: function() {
		// yield a call to hide the controls on blur of any footer link
		// if another footer link is the target of the focus transfer, the timeout will be cancelled
		this._footerExitTimeout = setTimeout(dojo.hitch(this,this._hideControls),0);
	},
	
	_footerLinkFocus: function() {
		// cancel the timeout to hide the footer since we are still inside it
		clearTimeout(this._footerExitTimeout);
	},
	
	refresh: function() {
		this._entries = new Array();
		this._details = new Array();
		this._clearHeight();
		this._currentWidth = this._contentArea.offsetWidth;
		this.update();
	},
	
	_onZoom: function() {
		// don't auto-refresh while zoomed
		this.getSite().setAutoRefresh(false);
	},
	
	_onUnzoom: function() {
		// resume auto-refresh
		this.getSite().setAutoRefresh(true);
	},
	
	_render: function() {
		if (this.getResultCount() == 0) {
			this._renderNoResults();
			this._setLoading(false);
			return;
		}
		var itemsPerPage = this.getItemsPerPage();
		var startIndex = (this._currentPage - 1) * itemsPerPage;
		var endIndex = startIndex + itemsPerPage;
		if (!this._pageWidget) {
			this._pageWidget = new com.ibm.team.dashboard.web.ui.PagedTableViewlet._Page({viewlet: this});
			if (this._contentArea.firstChild)
				this._contentArea.replaceChild(this._pageWidget.domNode, this._contentArea.firstChild);
			else
				this._contentArea.appendChild(this._pageWidget.domNode);
			while(this._contentArea.firstChild.nextSibling)
				this._contentArea.removeChild(this._contentArea.firstChild.nextSibling);
			this._pageWidget.setContent(this._entries.slice(startIndex, endIndex), this.isZoomed()? this._details.slice(startIndex, endIndex) : null, this.isZoomed());
		} else {
			if (!this.isZoomed() && !this._pageWidget.isZoomed() && !this._currentHeight && this._pageWidget.domNode.offsetHeight > 0) {
				this._currentHeight = this._pageWidget.domNode.offsetHeight;
				this._contentArea.style.height = this._currentHeight + "px";
			}
			this._pageWidget.setContent(this._entries.slice(startIndex, endIndex), this.isZoomed()? this._details.slice(startIndex, endIndex) : null, this.isZoomed());
			if (!this.isZoomed() && this._currentHeight && this._pageWidget.domNode.offsetHeight > this._currentHeight) {
				this._currentHeight = this._pageWidget.domNode.offsetHeight;
				this._contentArea.style.height = this._currentHeight + "px";
			}
		}
	},
	
	_onResize: function() {
		if (this._currentWidth == this._contentArea.offsetWidth)
			return;
		this._currentWidth = this._contentArea.offsetWidth;
		this._clearHeight();
	},
	
	_clearHeight: function() {
		this._contentArea.style.height = "";
		delete this._currentHeight;
		if (this._shrinkTimer) {
			clearTimeout(this._shrinkTimer);
			delete this._shrinkTimer;
		}
	},
	
	_renderNoResults: function() {
		this._showMessage(this.getNoResultsMessage());
	},

	_showMessage: function(/* Node */ node) {
		while(this._contentArea.firstChild)
			this._contentArea.removeChild(this._contentArea.firstChild);
		if (this._pageWidget) {
			this._pageWidget.destroy();
			delete this._pageWidget;
		}
		var messageDiv = document.createElement("div");
		messageDiv.className = "message";
		messageDiv.appendChild(node);
		this._contentArea.appendChild(messageDiv);
		this._clearHeight();
	},
	
	_doShrinkAnimation: function() {
		if (!this._pageWidget) {
			this._shrinkTimer = setTimeout(dojo.hitch(this,this._doShrinkAnimation), 100);
			return;
		}
		if (!this._currentHeight)
			return;
		delete this._shrinkTimer;
		if (!this.getSite().isActive()) {
			delete this._currentHeight;
			this._contentArea.style.height = "";
			return;
		}
		var endHeight = this._pageWidget.domNode.offsetHeight;
		var anim = dojo.animateProperty({node: this._contentArea, duration: 350, properties: {height: {start: this._currentHeight, end: endHeight, unit: "px"}}});
		delete this._currentHeight;
		this.connect(anim, "onEnd", function(){
			this._contentArea.style.height = "";
		});
		setTimeout(function(){anim.play();}, 0);
	},
	
	_handleNextPage: function(event) {
		dojo.stopEvent(event);
		if (this._currentPage == this._getPageCount()) {
			return;
		}
		
		this._currentPage++;
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_handlePrevPage: function(event) {
		dojo.stopEvent(event);
		if (this._currentPage == 1) {
			return;
		}
			
		this._currentPage--;
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_handleFirstPage: function(event) {
		dojo.stopEvent(event);
		this._first.blur();
		if (this._currentPage == 1) {
			return;
		}

		this._currentPage = 1;
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_handleLastPage: function(event) {
		dojo.stopEvent(event);
		this._last.blur();
		if (this._currentPage == this._getPageCount()) {
			return;
		}

		this._currentPage = this._getPageCount();
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_setLoading: function(/* boolean */ visible) {
		if (visible) {
			dojo.removeClass(this._loadingDiv, "hidden");
			dojo.addClass(this._contentArea, "hidden");
			dojo.addClass(this._globalHeading, "hidden");
			dojo.addClass(this._footer, "hidden");
		} else {
			dojo.addClass(this._loadingDiv, "hidden");
			dojo.removeClass(this._contentArea, "hidden");
			dojo.removeClass(this._globalHeading, "hidden");
		}
	},
	
	_getPageCount: function() {
		return Math.ceil(this.getResultCount() / this.getItemsPerPage());
	},
	
	_showControls: function() {
		if (this._currentPage != 1) {
			this._firstLinkImage.src = this.firstLinkImageSrc;
			this._firstLinkDisabled = false;
			this._firstLink.tabIndex = 0;
			if (this._prev.firstChild != this._prevLink) {
				this._prev.replaceChild(this._prevLink, this._prev.firstChild);
			}
		} else {
			this._firstLinkImage.src = dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/paged-table-viewlet/images/beginning-disabled.gif");
			this._firstLinkDisabled = true;
			this._firstLink.tabIndex = -1;
			if (this._prev.firstChild == this._prevLink) {
				var label = document.createElement("span");
				label.className = "disabled";
				label.innerHTML = messages.pagedTablePrev;
				this._prev.replaceChild(label, this._prev.firstChild);
			}
		}
		if (this._currentPage != this._getPageCount()) {
			this._lastLinkImage.src = this.lastLinkImageSrc;
			this._lastLinkDisabled = false;
			this._lastLink.tabIndex = 0;
			if (this._next.lastChild != this._nextLink) {
				this._next.replaceChild(this._nextLink, this._next.lastChild);
			}
		} else {
			this._lastLinkImage.src = dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/paged-table-viewlet/images/end-disabled.gif");
			this._lastLinkDisabled = true;
			this._lastLink.tabIndex = -1;
			if (this._next.lastChild == this._nextLink) {
				var label = document.createElement("span");
				label.className = "disabled";
				label.innerHTML = messages.pagedTableNext;
				this._next.replaceChild(label, this._next.lastChild);
			}
		}
		dojo.removeClass(this._prev, "invisible");
		dojo.removeClass(this._next, "invisible");
		dojo.removeClass(this._first, "invisible");
		dojo.removeClass(this._last, "invisible");
		if (this._shrinkTimer) {
			clearTimeout(this._shrinkTimer);
			delete this._shrinkTimer;
		}
	},
	
	_hideControls: function(e) {
		if (e && dojo.isDescendant(e.relatedTarget, this.domNode.parentNode) == true)
			return;
		dojo.addClass(this._prev, "invisible");
		dojo.addClass(this._next, "invisible");
		dojo.addClass(this._first, "invisible");
		dojo.addClass(this._last, "invisible");
		this._footerStart.tabIndex = 0;
		this._footerEnd.tabIndex = 0;
		if (!this.isZoomed())
			this._shrinkTimer = setTimeout(dojo.hitch(this,this._doShrinkAnimation), 2500);
	},
	
	_onMouseOverButton: function (e) {
		if (e.target == this._firstLink || e.target == this._firstLinkImage) {
			this.firstLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/paged-table-viewlet/images/beginning-hov.gif");
			if (!this._firstLinkDisabled)
				this._firstLinkImage.src = this.firstLinkImageSrc;
		} else {
			this.lastLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/paged-table-viewlet/images/end-hov.gif");
			if (!this._lastLinkDisabled)
				this._lastLinkImage.src = this.lastLinkImageSrc;
		}
	},
	
	_onMouseOutButton: function (e) {
		if (e.target == this._firstLink || e.target == this._firstLinkImage) {
			if (dojo.isDescendant(e.relatedTarget, this._firstLink) == true)
				return;
			this.firstLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/paged-table-viewlet/images/beginning.gif");
			if (!this._firstLinkDisabled)
				this._firstLinkImage.src = this.firstLinkImageSrc;
		} else {
			if (dojo.isDescendant(e.relatedTarget, this._lastLink) == true)
				return;
			this.lastLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/paged-table-viewlet/images/end.gif");
			if (!this._lastLinkDisabled)
				this._lastLinkImage.src = this.lastLinkImageSrc;
		}
	},
	
	_updateCurrentPageFooter: function() {
		this._currentPageFooter.innerHTML = this._currentPage;
	},
	
	destroy: function() {
		if (this._pageWidget)
			this._pageWidget.destroy();
		ResizeNotifier.unregister(this._dashboardResizeEvent);
		this.onDestroy();
		this.inherited(arguments);
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.PagedTableViewlet._Page",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<table class=\"pageTable\" cellspacing=\"0\"><tbody dojoAttachPoint=\"tbody\"></tbody></table>",
	
	constructor: function(args) {
		this._viewlet = args.viewlet;
	},
	
	isZoomed: function() {
		return this._isZoomed;
	},
	
	setContent: function(/* object[] */ entries, /* object[] */ details, /* boolean */ isZoomed) {
		// this._currentNode is used to replace existing table nodes
		this._isZoomed = isZoomed;
		this._currentNode = this.tbody.firstChild;
		while (this._currentNode && this._currentNode.nodeName == "#text")
			this._currentNode = this._currentNode.nextSibling;
		this._viewlet.preRender(entries);
		for (var i = 0; i < entries.length; i++) {
			this._doAppendReplaceNode(this._createRow(this._viewlet.renderEntry(entries[i], isZoomed ? details[i] : null, isZoomed)));
			if (details && details[i]) {
				this._doAppendReplaceNode(this._createRow(this._viewlet.renderDetails(details[i])));
			}
		}
		// remove any remaining old rows after populating the table
		while(this._currentNode) {
			var removeNode = this._currentNode;
			this._currentNode = this._currentNode.nextSibling;
			this.tbody.removeChild(removeNode);
		}
	},
	
	_createRow: function(/* Node[] */ columns) {
		var tr = document.createElement("tr");
		for (var i=0;i<columns.length;i++) {
			tr.appendChild(columns[i]);
		}
		return tr;
	},

	_doAppendReplaceNode: function(node) {
		if (this._currentNode) {
			var replaceNode = this._currentNode;
			this.tbody.replaceChild(node, replaceNode);
			this._currentNode = node.nextSibling;
		}
		else {
			this.tbody.appendChild(node);
		}
	},
	
	destroy: function() {
		while (this.tbody.firstChild)
			this.tbody.removeChild(this.tbody.firstChild);
	}

});

})();
